
--dropSlot stuff
--there is a 4 x 5 grid of dropSlots which is stored in an array dropSlotArray
--each dropSlot will have a texture that matches the spell, item or macro texture or icon in that slot
--each dropSlot has  properties
--dropSlot.theType spell, item or macro
--dropSlot.theID   global spellID,  itemID or macroID
--dropSlot.keyPress   control, shift option, alt
--dropSlot.mouseButton left center right
--dropSlot.actionWhenPressed - this will usually be a spell name. That gets loaded into the unit buttons
--dropSlot.name - not really used, just a placeholder
--dropSlot.slotNumber - 1-20. I need this so when passing arround a dropSlot, I know what slot # it is

--define misc local variables
local x,y;

--Define name space for all WhammyConfig functions
WhammyConfig = {}
WhammyConfig.dropSlotArray = {}; --array that stores the drop slots

--craft the whammyConfig frame
local whammyConfigFrame = CreateFrame("FRAME", "WhammyConfigFrame", UIParent)



--this is called from the main whammy frame to set up the config window once the vars are loaded
function WhammyConfig:Init()
	WhammyConfig:FrameCreate()
	WhammyConfig:FrameCreateControls()
	WhammyConfig:FrameCreateDropSlots()
	
	WhammyConfig:UpdateControlStatus();
end

--called from init and also after reset to update the control status
function WhammyConfig:UpdateControlStatus()
	
	whammyConfigFrame.muteAilmentSoundCheckButton:SetChecked( W_Options.Mute );	
	whammyConfigFrame.summaryCheckButton:SetChecked( W_Options.ShowSummary );
	if ( W_Options.ButtonSize == Whammy.constants.kButtonSizeStandard ) then -- I left the button size as an integer in case we want to add a third button size in the future
		whammyConfigFrame.buttonSizeCheckButton:SetChecked( false ); --true means small buttons, false is larger buttons
	elseif ( W_Options.ButtonSize == Whammy.constants.kButtonSizeSmall ) then
		whammyConfigFrame.buttonSizeCheckButton:SetChecked( true );
	end
	whammyConfigFrame.rangeCheckButton:SetChecked( W_Options.RangeCheck );
	whammyConfigFrame.threatNotifyButton:SetChecked( W_Options.ThreatNotify );
	whammyConfigFrame.isFrameMovableButton:SetChecked( W_Options.Movable );
	whammyConfigFrame.showDropdownButton:SetChecked( W_Options.ShowDropdown );
	
	UIDropDownMenu_SetSelectedName(WC_Health_dd1, W_Options.Health1, true); --set the popup selection (not the text)
	UIDropDownMenu_SetSelectedName(WC_Health_dd2, W_Options.Health2, true); --set the popup selection (not the text)
	UIDropDownMenu_SetText(WC_Health_dd1, W_Options.Health1, true); --and set the text on the popup
	UIDropDownMenu_SetText(WC_Health_dd2, W_Options.Health2, true);
	
	
	if ( W_Options.Health1 ~= "Off" ) then --if health1 is not off, activate health 2
		UIDropDownMenu_EnableDropDown( WC_Health_dd2 );
		WC_Health2_Text:SetFontObject( "GameFontNormalSmall");
	else --health dd is off so disable dd2
		UIDropDownMenu_DisableDropDown( WC_Health_dd2 );
		WC_Health2_Text:SetFontObject( "GameFontDisableSmall");
	end
	
	WhammyConfig:Click_CheckButton_Summary( W_Options.ShowSummary );	-- show the summary frame or not
	WhammyConfig:Click_CheckButton_RangeCheck( W_Options.RangeCheck ) --init the range checking to on or off
	whammyConfigFrame.miniMapSlider:SetValue(W_Options.Angle); --set the slider on the config window
	
end


function WhammyConfig:FrameCreate()
	
	local strata = "ARTWORK"
	
	whammyConfigFrame:SetFrameStrata("MEDIUM") --this frame will appear in front of the main whammy frame, which is set to LOW
	whammyConfigFrame:SetClampedToScreen(true)

	whammyConfigFrame:SetWidth(600)
	whammyConfigFrame:SetHeight(400)
	whammyConfigFrame:SetPoint("CENTER", "UIParent", "CENTER")
	whammyConfigFrame:SetResizable(false)
	whammyConfigFrame:EnableMouse(true)
	whammyConfigFrame:SetMovable(true)

	--set up a title region in the frame used for dragging
 	whammyConfigFrame.titleRegion = whammyConfigFrame:CreateTitleRegion()
	whammyConfigFrame.titleRegion:SetSize(600, 24)
	whammyConfigFrame.titleRegion:SetPoint("TOPLEFT", whammyConfigFrame, "TOPLEFT")
	--this is the drag bar texture
	whammyConfigFrame.titleTexture = whammyConfigFrame:CreateTexture("WhammyConfigTitleTexture", strata)
	whammyConfigFrame.titleTexture:SetSize(600, 24)
	whammyConfigFrame.titleTexture:SetPoint("TOPLEFT")
	whammyConfigFrame.titleTexture:SetTexture( .3, .3 , .3, .5)
	
	whammyConfigFrame:Hide()

	--the texture of the config frame... a very very light grey
	whammyConfigFrame.texture = whammyConfigFrame:CreateTexture("WhammyConfigFrameTexture", strata)
	whammyConfigFrame.texture:SetTexture(.1,.1,.1,.4)
	whammyConfigFrame.texture:SetAllPoints(whammyConfigFrame)

	local tt = CreateFrame("GameTooltip", "MyScanningTooltip", nil, "GameTooltipTemplate") --tooltip
	tt:SetOwner( whammyConfigFrame, "ANCHOR_TOPRIGHT" );
	
	whammyConfigFrame.closeButton = CreateFrame("Button", "WC_CloseButton", whammyConfigFrame, "UIPanelCloseButton");
	whammyConfigFrame.closeButton:SetPoint("TOPRIGHT", whammyConfigFrame, "TOPRIGHT", 0, 4);

	-- Create the window title
	x = 0
	y = -6
	whammyConfigFrame.title = whammyConfigFrame:CreateFontString("WC_Title", strata, "GameFontNormal");
	whammyConfigFrame.title:SetText( Whammy.localize.kcConfigWindowTitle );
	whammyConfigFrame.title:SetPoint("TOP", whammyConfigFrame, "TOP", 0, y); --centered at the top

	-- Create the Mouse Button Clicks Title
	x = 130;
	y = -35
	whammyConfigFrame.mouseClicksTitle = whammyConfigFrame:CreateFontString("WC_Title_MouseClicks", strata, "GameFontNormal");
	whammyConfigFrame.mouseClicksTitle:SetText(Whammy.localize.kcMouseClicksText);
	whammyConfigFrame.mouseClicksTitle:SetPoint("TOPLEFT", whammyConfigFrame, "TOPLEFT", x, y);

	-- Create the Other Options Title
	x = 430;
	whammyConfigFrame.otherOptionsTitle = whammyConfigFrame:CreateFontString("WC_Title_OtherOptions", strata, "GameFontNormal");
	whammyConfigFrame.otherOptionsTitle:SetText(Whammy.localize.kcOtherOptionsText);
	whammyConfigFrame.otherOptionsTitle:SetPoint("TOPLEFT", whammyConfigFrame, "TOPLEFT", x, y);

	-- Create the Left, Middle, Right labels down the left hand side
	x = 30;
	y = -95;
	whammyConfigFrame.left = whammyConfigFrame:CreateFontString("WC_Label_Left", strata, "GameFontNormalSmall");
	whammyConfigFrame.left:SetText(Whammy.localize.kcLeftText);
	whammyConfigFrame.left:SetPoint("TOPLEFT", whammyConfigFrame, "TOPLEFT", x, y);

	y=y-45;
	whammyConfigFrame.middle = whammyConfigFrame:CreateFontString("WC_Label_Center", strata, "GameFontNormalSmall");
	whammyConfigFrame.middle:SetText(Whammy.localize.kcMiddleText);
	whammyConfigFrame.middle:SetPoint("TOPLEFT", whammyConfigFrame, "TOPLEFT", x, y);

	y=y-45;
	whammyConfigFrame.right = whammyConfigFrame:CreateFontString("WC_Label_Right", strata, "GameFontNormalSmall");
	whammyConfigFrame.right:SetText(Whammy.localize.kcRightText);
	whammyConfigFrame.right:SetPoint("TOPLEFT", whammyConfigFrame, "TOPLEFT", x, y);

	y=y-45;
	whammyConfigFrame.button4 = whammyConfigFrame:CreateFontString("WC_Label_Button4", strata, "GameFontNormalSmall");
	whammyConfigFrame.button4:SetText(Whammy.localize.kcButton4Text);
	whammyConfigFrame.button4:SetPoint("TOPLEFT", whammyConfigFrame, "TOPLEFT", x, y);

	y=y-45;
	whammyConfigFrame.button5 = whammyConfigFrame:CreateFontString("WC_Label_Button5", strata, "GameFontNormalSmall");
	whammyConfigFrame.button5:SetText(Whammy.localize.kcButton5Text);
	whammyConfigFrame.button5:SetPoint("TOPLEFT", whammyConfigFrame, "TOPLEFT", x, y);

	-- Create the No modifier, Shift, Ctrl and Alt labels across the top
	x = 96;
	y = -65;
	whammyConfigFrame.shift = whammyConfigFrame:CreateFontString("WC_Label_NoMod", strata, "GameFontNormalSmall");
	whammyConfigFrame.shift:SetText(Whammy.localize.kcNoMod);
	whammyConfigFrame.shift:SetPoint("TOPLEFT", whammyConfigFrame, "TOPLEFT", x, y)

	x=170
	whammyConfigFrame.shift = whammyConfigFrame:CreateFontString("WC_Label_Shift", strata, "GameFontNormalSmall");
	whammyConfigFrame.shift:SetText(Whammy.localize.kcShift);
	whammyConfigFrame.shift:SetPoint("TOPLEFT", whammyConfigFrame, "TOPLEFT", x, y)

	x=230
	whammyConfigFrame.ctrl = whammyConfigFrame:CreateFontString("WC_Label_Ctrl", strata, "GameFontNormalSmall");
	whammyConfigFrame.ctrl:SetText(Whammy.localize.kcCtrl);
	whammyConfigFrame.ctrl:SetPoint("TOPLEFT", whammyConfigFrame, "TOPLEFT", x, y)

	x=290
	whammyConfigFrame.alt = whammyConfigFrame:CreateFontString("WC_Label_Alt", strata, "GameFontNormalSmall");
	whammyConfigFrame.alt:SetText(Whammy.localize.kcAlt);
	whammyConfigFrame.alt:SetPoint("TOPLEFT", whammyConfigFrame, "TOPLEFT", x, y)


	
	
end

function WhammyConfig:FrameCreateControls()
	local x = 10;
	local y = -20;
	
	--Create a divider to separate one half from the other. It's also a point of reference for the checkboxes
	whammyConfigFrame.divider = whammyConfigFrame:CreateTexture();
	whammyConfigFrame.divider:SetWidth(2);
	local h = whammyConfigFrame:GetHeight()-60; --height of the frame - 40, 20 space on top and bottom
	whammyConfigFrame.divider:SetHeight(h);
	whammyConfigFrame.divider:SetPoint("TOPLEFT", whammyConfigFrame, "TOPLEFT", h, -30);
	whammyConfigFrame.divider:SetTexture(.5, .5, .5, .4);

	
	--Create and Setup Defaults Text and Button
	whammyConfigFrame.defaultText = whammyConfigFrame:CreateFontString("WC_Set_Defaults_Text", strata, "GameFontNormal");
	whammyConfigFrame.defaultText:SetJustifyH("LEFT")
	whammyConfigFrame.defaultText:SetText(Whammy.localize.kcSetDefaultText);
	whammyConfigFrame.defaultText:SetWidth(340);
	whammyConfigFrame.defaultText:SetPoint("BOTTOMLEFT", whammyConfigFrame, "BOTTOMLEFT", 10, 50);
	
	whammyConfigFrame.defaultButton = CreateFrame("Button", "WC_DefaultButton", whammyConfigFrame, "UIPanelButtonTemplate");
	whammyConfigFrame.defaultButton:SetWidth(80);
	whammyConfigFrame.defaultButton:SetHeight(22);
	whammyConfigFrame.defaultButton:SetPoint("TOPLEFT", whammyConfigFrame.defaultText, "TOPLEFT", 240, -35);
	whammyConfigFrame.defaultButton:SetScript("OnClick", function(self) WhammyConfig:DefaultButton_OnClick(self); end );
	whammyConfigFrame.defaultButton:SetText("Defaults");

	--Create and setup Summary CheckButton (the little summary window that shows group mana and health)
	whammyConfigFrame.summaryCheckButton = CreateFrame("CheckButton", "WC_CheckButton_ShowSummary", whammyConfigFrame, "UICheckButtonTemplate");
	whammyConfigFrame.summaryCheckButton:SetWidth(30);
	whammyConfigFrame.summaryCheckButton:SetHeight(30);
	whammyConfigFrame.summaryCheckButton:SetPoint("TOPLEFT", whammyConfigFrame.divider, "TOPRIGHT", x, y);
	whammyConfigFrame.summaryCheckButton:SetScript("OnClick", function(self) WhammyConfig:Click_CheckButton_Summary( self:GetChecked() ); end );
	getglobal(whammyConfigFrame.summaryCheckButton:GetName().."Text"):SetText(Whammy.localize.kcShowSummaryCheckButtonText);
	
	--Create and setup the Button Size CheckButton
	whammyConfigFrame.buttonSizeCheckButton = CreateFrame("CheckButton", "WC_CheckButton_ButtonSize", whammyConfigFrame, "UICheckButtonTemplate");
	whammyConfigFrame.buttonSizeCheckButton:SetWidth(30);
	whammyConfigFrame.buttonSizeCheckButton:SetHeight(30);
	whammyConfigFrame.buttonSizeCheckButton:SetPoint("TOPLEFT", whammyConfigFrame.divider, "TOPRIGHT", x, y-25);
	whammyConfigFrame.buttonSizeCheckButton:SetScript("OnClick", function(self) WhammyConfig:Click_CheckButton_ButtonSize( self:GetChecked() ); end );
	getglobal(whammyConfigFrame.buttonSizeCheckButton:GetName().."Text"):SetText(Whammy.localize.kcButtonSizeCheckButtonText);

	--Create and setup the Mute Ailment Sound CheckButton
	whammyConfigFrame.muteAilmentSoundCheckButton = CreateFrame("CheckButton", "WC_CheckButton_MuteAilmentSound", whammyConfigFrame, "UICheckButtonTemplate");
	whammyConfigFrame.muteAilmentSoundCheckButton:SetWidth(30);
	whammyConfigFrame.muteAilmentSoundCheckButton:SetHeight(30);
	whammyConfigFrame.muteAilmentSoundCheckButton:SetPoint("TOPLEFT", whammyConfigFrame.divider, "TOPRIGHT", x, y-50);
	whammyConfigFrame.muteAilmentSoundCheckButton:SetScript("OnClick", function(self) WhammyConfig:Click_CheckButton_MuteAilmentSound( self:GetChecked() ); end );
	getglobal(whammyConfigFrame.muteAilmentSoundCheckButton:GetName().."Text"):SetText(Whammy.localize.kcMuteAilmentSoundText);

	--Create and setup the Range Check CheckButton
	whammyConfigFrame.rangeCheckButton = CreateFrame("CheckButton", "WC_CheckButton_RangeCheck", whammyConfigFrame, "UICheckButtonTemplate");
	whammyConfigFrame.rangeCheckButton:SetWidth(30)
	whammyConfigFrame.rangeCheckButton:SetHeight(30)
	whammyConfigFrame.rangeCheckButton:SetPoint("TOPLEFT", whammyConfigFrame.divider, "TOPRIGHT", x, y-75)
	whammyConfigFrame.rangeCheckButton:SetScript("OnClick", function(self) WhammyConfig:Click_CheckButton_RangeCheck( self:GetChecked() ); end );
	getglobal(whammyConfigFrame.rangeCheckButton:GetName().."Text"):SetText(Whammy.localize.kcRangeCheckText);
	
	--Create and setup the Threat notification CheckButton
	whammyConfigFrame.threatNotifyButton = CreateFrame("CheckButton", "WC_CheckButton_ThreatNotify", whammyConfigFrame, "UICheckButtonTemplate");
	whammyConfigFrame.threatNotifyButton:SetWidth(30)
	whammyConfigFrame.threatNotifyButton:SetHeight(30)
	whammyConfigFrame.threatNotifyButton:SetPoint("TOPLEFT", whammyConfigFrame.divider, "TOPRIGHT", x, y-100)
	whammyConfigFrame.threatNotifyButton:SetScript("OnClick", function(self) WhammyConfig:Click_CheckButton_ThreatNotify( self:GetChecked() ); end );
	getglobal(whammyConfigFrame.threatNotifyButton:GetName().."Text"):SetText(Whammy.localize.kcThreatNotifyText);
	
	--Create and setup the Lock Frame CheckButton
	whammyConfigFrame.isFrameMovableButton = CreateFrame("CheckButton", "WC_CheckButton_IsFrameMovable", whammyConfigFrame, "UICheckButtonTemplate");
	whammyConfigFrame.isFrameMovableButton:SetWidth(30)
	whammyConfigFrame.isFrameMovableButton:SetHeight(30)
	whammyConfigFrame.isFrameMovableButton:SetPoint("TOPLEFT", whammyConfigFrame.divider, "TOPRIGHT", x, y-125)
	whammyConfigFrame.isFrameMovableButton:SetScript("OnClick", function(self) WhammyConfig:Click_CheckButton_IsFrameMovable( self:GetChecked() ); end );
	getglobal(whammyConfigFrame.isFrameMovableButton:GetName().."Text"):SetText(Whammy.localize.kcIsFrameMovableText);
	
	--Create and setup the Show Dropdown for umapped buttons CheckButton
	whammyConfigFrame.showDropdownButton = CreateFrame("CheckButton", "WC_CheckButton_ShowDropdown", whammyConfigFrame, "UICheckButtonTemplate");
	whammyConfigFrame.showDropdownButton:SetWidth(30)
	whammyConfigFrame.showDropdownButton:SetHeight(30)
	whammyConfigFrame.showDropdownButton:SetPoint("TOPLEFT", whammyConfigFrame.divider, "TOPRIGHT", x, y-150)
	whammyConfigFrame.showDropdownButton:SetScript("OnClick", function(self) WhammyConfig:Click_CheckButton_ShowDropdown( self:GetChecked() ); end );
	getglobal(whammyConfigFrame.showDropdownButton:GetName().."Text"):SetText(Whammy.localize.kcShowDropdownText);
	
	-- The labels and dropdowns for the two health alert dropdowns
	whammyConfigFrame.healthText = whammyConfigFrame:CreateFontString("WC_Health_Text", strata, "GameFontNormalSmall");
	whammyConfigFrame.healthText:SetText(Whammy.localize.kcConfigHealthAlertText);
	whammyConfigFrame.healthText:SetPoint("TOPLEFT", whammyConfigFrame.divider, "TOPRIGHT", x+5, y-190);
	
	local ddName = "WC_Health_dd1";
	local items = {"Off","10","20","30","40","50","60","70","80","90"}; --table for the percentage health
	local mydd1 = WhammyConfig:CreateDropDownMenu( ddName,  items )
	mydd1:SetPoint("LEFT", whammyConfigFrame.healthText, "RIGHT", -10, 0)
	mydd1:Show()

	whammyConfigFrame.health2Text = whammyConfigFrame:CreateFontString("WC_Health2_Text", strata, "GameFontDisableSmall");
	whammyConfigFrame.health2Text:SetText(Whammy.localize.kcConfigHealthAlertText2);
	whammyConfigFrame.health2Text:SetPoint("TOPLEFT", whammyConfigFrame.divider, "TOPRIGHT", x+35, y-220);
	
	local ddName2 = "WC_Health_dd2";
	local mydd2 = WhammyConfig:CreateDropDownMenu( ddName2,  items )
	mydd2:SetPoint("LEFT", whammyConfigFrame.health2Text, "RIGHT", -10, 0)
	mydd2:Show()
	
	
	--Create and position the minimap icon text and slider in the config window
	whammyConfigFrame.sliderText = whammyConfigFrame:CreateFontString("WC_Slider_Text", strata, "GameFontNormalSmall");
	whammyConfigFrame.sliderText:SetJustifyH("LEFT");
	whammyConfigFrame.sliderText:SetText( Whammy.localize.kcIcon_Pos_Text );
	whammyConfigFrame.sliderText:SetWidth(100);
	whammyConfigFrame.sliderText:SetPoint("TOPLEFT", whammyConfigFrame.divider, "TOPRIGHT", x+5, y-250);
	
	whammyConfigFrame.miniMapSlider = CreateFrame("Slider", "WC_MiniMapIconPosSlider", whammyConfigFrame, "OptionsSliderTemplate")
	whammyConfigFrame.miniMapSlider:SetWidth( 100 );
	whammyConfigFrame.miniMapSlider:SetHeight( 24 );
	whammyConfigFrame.miniMapSlider:SetPoint("TOPLEFT", whammyConfigFrame.sliderText, "TOPRIGHT", 15, 5)
	whammyConfigFrame.miniMapSlider:SetMinMaxValues(0,359);
	whammyConfigFrame.miniMapSlider:SetValueStep(1);
	whammyConfigFrame.miniMapSlider:SetScript("OnValueChanged", function(self) WhammyConfig:MiniMapIconPosSlider_ValueChanged(self); end );
	
	WC_MiniMapIconPosSliderText:SetFont("FRIZQT__.TTF",11,"OUTLINE, MONOCHROME");
	WC_MiniMapIconPosSliderText:SetText();
	WC_MiniMapIconPosSliderText:SetTextColor(0,1,1);
	WC_MiniMapIconPosSliderHigh:SetText();
	WC_MiniMapIconPosSliderLow:SetText();
end

function WhammyConfig:FrameCreateDropSlots()

	local x = 0;
	local y = 0;
	
	local slot
	
	for i = 1,20 do
		
		local dropSlotName = "WC_DropSlot"..i;
		slot = CreateFrame("CheckButton", dropSlotName, whammyConfigFrame, "ItemButtonTemplate");
		slot.slotNumber = i;
		tinsert(WhammyConfig.dropSlotArray, slot); --insert the slot into the slot array
	
		slot:SetWidth(36);
		slot:SetHeight(36);
		slot:RegisterForDrag("LeftButton")
		slot:SetScript("OnReceiveDrag", function(self) WhammyConfig:DropSlot_Dropped(self); end );
		slot:SetScript("OnClick", function(self) WhammyConfig:DropSlot_OnClick(self); end );
		slot:SetScript("OnEnter", function(self) WhammyConfig:DropSlot_OnEnter(self); end);
		slot:SetScript("OnLeave", function(self) WhammyConfig:DropSlot_OnLeave(self); end);
		slot:SetScript("OnDragStart", function(self) WhammyConfig:DropSlot_OnDragStart(self); end);
		
		-- Button:SetDisabledTexture(texture) or Button:SetDisabledTexture("filename")
		-- Arguments:
		-- 
		-- texture - Reference to an existing Texture object (texture)
		-- filename - Path to a texture image file (string)
		slot:SetMotionScriptsWhileDisabled(true) --this is used when a dropSlot is disabled, on enter events are called so we can tell the user why it's disabled
		local greyTexture = slot:CreateTexture("DropSlotDisabledTexture", strata)
		greyTexture:SetSize(36, 36)
		greyTexture:SetPoint("TOPLEFT")
		greyTexture:SetTexture( .3, .3 , .3, .5)
		slot:SetDisabledTexture(greyTexture)
		
		if (i==6 or i==7 or i==8 or i==9 or i==10) then
			slot.keyPress = "shift";
		elseif (i==11 or i==12 or i==13 or i==14 or i==15) then
			slot.keyPress = "ctrl";
		elseif (i==16 or i==17 or i==18 or i==19 or i==20) then
			slot.keyPress = "alt"
		else
			slot.keyPress = ""; --covers 1,2,3
		end
		
		-- note that left button = 1,  right = 2 and middle = 3. The 'grid' of buttons goes 1-5 downthe first column, then 6-10 etc
		if ( i==1 or i==6 or i==11 or i==16) then --left click
			slot.mouseButton = "1";
		elseif (i==2 or i==7 or i==12 or i==17) then --center click
			slot.mouseButton = "3";
		elseif (i==3 or i==8 or i==13 or i==18) then --right click
			slot.mouseButton = "2";
		elseif (i==4 or i==9 or i==14 or i==19) then --button 4
			slot.mouseButton = "4";
		elseif (i==5 or i==10 or i==15 or i==20) then --button 5
			slot.mouseButton = "5";
		end
	end
	
	--no modifier key slots
	x=102
	y=-82
	for i = 1,5 do
		local slot = WhammyConfig.dropSlotArray[i]
		slot:SetPoint("TOPLEFT", whammyConfigFrame, "TOPLEFT", x, y);
		y=y-46
	end
	
	--shift modifier key slots
	x=164
	y=-82
	for i = 6,10 do
		local slot = WhammyConfig.dropSlotArray[i]
		slot:SetPoint("TOPLEFT", whammyConfigFrame, "TOPLEFT", x, y);
		y=y-46
	end
	
	--ctrl modifier key slots
	x=223
	y=-82
	for i = 11,15 do
		local slot = WhammyConfig.dropSlotArray[i]
		slot:SetPoint("TOPLEFT", whammyConfigFrame, "TOPLEFT", x, y);
		y=y-46
	end
	
	--alt modifier key slots
	x=282
	y=-82
	for i = 16,20 do
		local slot = WhammyConfig.dropSlotArray[i]
		slot:SetPoint("TOPLEFT", whammyConfigFrame, "TOPLEFT", x, y);
		y=y-46
	end

end


--
-- Mouse dragging
--
function WhammyConfig:MouseDown( self, ... )

	local arg1 = ...;
	if ( arg1 == "LeftButton" ) then -- drag the frame
		whammyConfigFrame:StartMoving();
	end
end

function WhammyConfig:MouseUp(self)
	whammyConfigFrame:StopMovingOrSizing();
end

--
-- handle drop down stuff
--
function WhammyConfig:CreateDropDownMenu( ddName, items )
	--pass in the frame to attach the dropdown to, the name, an table of items
	local dd = CreateFrame("Button", ddName, whammyConfigFrame, "UIDropDownMenuTemplate")

	dd:ClearAllPoints()
	
	local InitializeDropdown = function()
		local info = UIDropDownMenu_CreateInfo()
		for k,v in pairs(items) do
			info.text = v
			info.value = v
			info.func = function(self)
				UIDropDownMenu_SetSelectedID( dd, self:GetID() )
				WhammyConfig:DropDown_OnClick( dd, self )
			end
			info.checked = nil
			UIDropDownMenu_AddButton(info, 1)
		end
	end

	UIDropDownMenu_Initialize(dd, InitializeDropdown)
	UIDropDownMenu_SetWidth(dd, 100);
	UIDropDownMenu_SetButtonWidth(dd, 124)
	UIDropDownMenu_SetSelectedID(dd, 1)
	UIDropDownMenu_JustifyText(dd, "LEFT")
	
	return dd;
end

function WhammyConfig:DropDown_OnClick( dd, whichButton ) --this is called when something is selected in a health dropdown
	local ddName = dd:GetName();
	local ddID = whichButton:GetID(); --each drop down option is it's own button
	local ddText = whichButton:GetText(); --this is the text of the button, Off, 10, 20...90
	--local ddValue = whichButton.value; --value is key value pair - we dont use it anywhere.

	if ( ddName == "WC_Health_dd1") then --something was clicked in the first dropdown
		W_Options.Health1 = ddText; --store what was clicked Off, 10, 20 etc
		if ( ddText == "Off" ) then --if we select off in the first drop down, disable the the second one
			W_Options.Health2 = ddText; --set the second dropdown to Off
			UIDropDownMenu_SetSelectedName(WC_Health_dd2, ddText, true); --set the 2nd dropdown to Off as well
			UIDropDownMenu_DisableDropDown( WC_Health_dd2 ); --disable the 2nd dropdown
			WC_Health2_Text:SetFontObject( "GameFontDisableSmall");
		else -- some value was selected in the first dropdown, so enable the second
			UIDropDownMenu_EnableDropDown( WC_Health_dd2 );
			WC_Health2_Text:SetFontObject( "GameFontNormalSmall");
		end
	elseif (ddName == "WC_Health_dd2" ) then
		W_Options.Health2 = ddText;
	end
end

--
-- Called when we first load to put the saved spells into their respective slots. Can only be called AFTER player entes world since we need to read from the spell book.
--  also called when SPELLS_CHANGED occurs - usually when a priest changes chakra for example
--
function WhammyConfig:DropSlot_Init()
	
	--print("DropSlot_Init")
	
	local activeSpec = GetActiveSpecGroup(); --1 for primary spec, 2 for secondary

	for slotNum = 1,20 do
		
		local action;
		
		if ( activeSpec == 1 ) then --primary spec
			action = W_Options.Actions[slotNum]; --actions is a small table that holds two things; type and id.  type = spell, item or macro. id is spellID, itemID or macroID	
		else --secondary spec
			action = W_Options.Actions2[slotNum];
		end
		
		local dropSlot = WhammyConfig.dropSlotArray[slotNum]

		if (action) then --if there is an action going into the slot, configure the slot with the type and id
			--WhammyConfig:PrintActionInfo( slotNum )

			local theType = action[1];
			local theID = action[2]; --if the user has an older version, they may not have the correct data. If its not a number, ignore it

			if ( theType and tonumber(theID) ) then --if theres a type and id then see if it's already in the slot. if not, update the slot. If so, ignore it
				
				WhammyConfig:DropSlot_Config(dropSlot, theType, theID)

			else --if the type or id was not or not valid, clear it

				WhammyConfig:DropSlot_Clear(dropSlot);
			end
			
			
			
		else --no action so clear the slot

			WhammyConfig:DropSlot_Clear(dropSlot);

		end
		
	end

end


function WhammyConfig:PrintSlotInfo( dropSlot )
	
	local slotNum = dropSlot.slotNumber
	local theType = dropSlot.theType
	local theID = dropSlot.theID
	
	if ( slotNum ) then
		print("WhammyConfig: dropSlot info #: "..slotNum)
		
		if ( theType ) then
			print("  type: "..theType)
			
			if ( theID ) then
				print("  id: "..theID)
			end
		end
	end
	
end

function WhammyConfig:PrintActionInfo( slotNum )
	
	local action = W_Options.Actions[slotNum]
	
	if ( action ) then
		
		local theType = action[1]
		local theID = action[2]
		print("WhammyConfig: action info #: "..slotNum)
		
		if ( theType ) then
			print("  type: "..theType)
		end
		
		if ( theID ) then
			print("  id: "..theID)
		end
		
	else
		print("no action for slot: "..slotNum)
	end
	
end
--
-- Called when we drop someting on a dropSlot. Get the item from the cursor and assign it to the slot
--
function WhammyConfig:DropSlot_Dropped( dropSlot )
	--Whammy:ShowMessage("DropSlot_Dropped")
	
	if ( InCombatLockdown() ) then
		UIErrorsFrame:AddMessage(Whammy.localize.kcNoChangesInCombat, 1.0, 0.0, 0.0, 53, 5);
	else
			
		if ( dropSlot.locked == true ) then --if the slot we are dropping on is disabled, do not accept the drop as it has something in it
			do return end --note that when the mouse cursor goes over the slot, it enabled to provide feedback to user. So we have a .locked property
		end
		
		local oldType, oldID;
		
		local activeSpec = GetActiveSpecGroup(); --1 for primary spec, 2 for secondary

		local theType, data, subType, subData = GetCursorInfo()--get what the cursor is holding
		--type can be one of the following: battlepet companion equipmentset flyout guildbankmoney item macro merchant money petaction spell
		--data is the data of the type i.e. the companion number, itemID or the spellBookIndex, which is the index of the spell in the players spellbook
		--subType is a Secondary identifier for the data on the cursor; used only for certain types: (string)
		--   companion - "CRITTER" or "MOUNT", indicating whether the returned data is an index in the non-combat pet or mount list
		--   item - A complete hyperlink for the item
		--   spell - "spell" or "pet", indicating whether the returned data is an index in the player's or pet's spellbook
		--subData - Secondary data based on subType: (value)
		--   spell - Spell ID.
		
		if ( dropSlot.theType ) then --the slot we are dropping on already has something in it, so accept the drop and stick the existing item back into the cursor
			oldType = dropSlot.theType;
			oldID = dropSlot.theID;
		end
		
		if ( CursorHasSpell() ) then --verify it's a spell that's valid for Whammy

			local spellBookIndex = data; --the cursor will contain the index of the item in the spellbook
			local name, rank, icon, castingTime, minRange, maxRange, globalSpellID = GetSpellInfo(spellBookIndex, BOOKTYPE_SPELL) --globalSpellID is the actual spell id which we save as a savedVariable
			
			local isHarmful = IsHarmfulSpell(spellBookIndex, BOOKTYPE_SPELL)

			if ( isHarmful == true ) then --if it's harmful, we dont allow, unless is a placeholder spell. A chakra priest spell for example
				if ( globalSpellID == Whammy.constants.kChakraChastise ) then
					WhammyConfig:DropSlot_Config(dropSlot, theType, globalSpellID) --this will take the spell stored in the dropslot and assign it to the unit buttons
					WhammyConfig:DropSlot_AddThingToCursor( oldType, oldID) --determine what the old thing was and add it to the cursor. oldType may be nil so it will be ignored
				else
					UIErrorsFrame:AddMessage(Whammy.localize.kcNoHarmfulSpells, 1.0, 0.0, 0.0, 53, 5);
				end
				
			else --was not a harmful spell, so allow
				WhammyConfig:DropSlot_Config(dropSlot, theType, globalSpellID) --this will take the spell stored in the dropslot and assign it to the unit buttons
				WhammyConfig:DropSlot_AddThingToCursor( oldType, oldID) --determine what the old thing was and add it to the cursor. oldType may be nil so it will be ignored

			end
			
		elseif ( theType == "item" ) then

			local itemID = data

			WhammyConfig:DropSlot_Config(dropSlot, theType, itemID)
			WhammyConfig:DropSlot_AddThingToCursor( oldType, oldID) --determine what the old thing was and add it to the cursor. oldType may be nil so it will be ignored

		elseif ( CursorHasMacro() == true ) then

			local macroID = data;
			
			WhammyConfig:DropSlot_Config(dropSlot, theType, macroID)
			WhammyConfig:DropSlot_AddThingToCursor( oldType, oldID) --determine what the old thing was and add it to the cursor
			
		elseif ( theType == "companion" ) then
			UIErrorsFrame:AddMessage(Whammy.localize.kcNoPetActions, 1.0, 0.0, 0.0, 53, 5);
		elseif ( theType == "petaction" ) then --determine what the old thing was and add it to the cursor. oldType may be nil so it will be ignored
			UIErrorsFrame:AddMessage(Whammy.localize.kcNoPetActions, 1.0, 0.0, 0.0, 53, 5);
		end
		
	end
end

--
-- Pickup item. This puts an item on the cursor after something was dropped on a slot that had something in it. Need to determine what the thing is and then pick it up
--
function WhammyConfig:DropSlot_AddThingToCursor( theType, theID)
	--Whammy:ShowMessage("WhammyConfig:DropSlot_AddThingToCursor")
	ClearCursor() --need to clear the cursor out before adding something to it
	
	if ( theType and theID ) then
		
		if ( theType == "spell" ) then

			local index = WhammyConfig:GetSpellBookIndexForGlobalSpellID( theID )
			
			PickupSpellBookItem( index, BOOKTYPE_SPELL)

		elseif ( theType == "item" ) then

			PickupItem(theID);
		
		elseif ( theType == "macro" ) then

			PickupMacro(theID);
		end
	end
	
end


-- Whammy.Chakra[1] = 88625 --Chastise
-- Whammy.Chakra[2] = 88685 --holy word: Sanctuary
-- Whammy.Chakra[3] = 88684 --holy word: Serenity
-- Whammy.Chakra[4] = 20711 --Spirit of Redemption

--
-- called when we drop a new spell, item or macro on a slot. When passed in the dropSlot should have it;s id property set to spell, item or macro, and then the global id of the spell item or macro
--
function WhammyConfig:DropSlot_Config(dropSlot, theType, theID)
	
	--print("DropSlot_Config")
	
	dropSlot:Enable() --start off being enabled
	dropSlot.locked = false
	
	if ( theType == "spell" ) then
		local globalSpellID = theID
		local spellName, rank, icon, castingTime, minRange, maxRange, spellID = GetSpellInfo(globalSpellID)
		
		SetItemButtonTexture( dropSlot, icon ); --set the texture of the slot
		
		if ( WhammyConfig.tableContains( Whammy.Chakra, globalSpellID) ) then
			local index = WhammyConfig:GetSpellBookIndexForGlobalSpellID( globalSpellID )

			if ( index == nil ) then
				dropSlot:Disable() --the spell in this slot is NOT available in the spellbook
				dropSlot.locked = true;
			end
		end
	
		dropSlot.actionWhenPressed = spellName --what to do when we click. This is then used to set up the unitButtons
		dropSlot.theType = theType --save this for later
		dropSlot.theID = globalSpellID
	
		WhammyConfig:UpdateUnitButtons( dropSlot ); --will update all the unit buttons with this new spell, item or macro
		
	elseif ( theType == "item" ) then
		local itemID = theID;
		local name, link, quality, iLevel, reqLevel, class, subclass, maxStack, equipSlot, texture, vendorPrice = GetItemInfo(itemID)
		
		SetItemButtonTexture( dropSlot, texture ); --set the texture of the slot
		dropSlot.actionWhenPressed = name --what to do when we click. This is then used to set up the unitButtons
		dropSlot.theType = theType --save this for later
		dropSlot.theID = itemID
		
		WhammyConfig:UpdateUnitButtons( dropSlot ); --will update all the unit buttons with this new spell, item or macro
		
		
	elseif ( theType == "macro" ) then
		local macroID = theID;
		local name, texture, body = GetMacroInfo(macroID)
		
		SetItemButtonTexture( dropSlot, texture ); --set the texture of the slot
		dropSlot.actionWhenPressed = name --what to do when we click. This is then used to set up the unitButtons
		dropSlot.actionWhenPressed = name --what to do when we click. This is then used to set up the unitButtons
		dropSlot.theType = theType --save this for later
		
		WhammyConfig:UpdateUnitButtons( dropSlot ); --will update all the unit buttons with this new spell, item or macro
	end
	
	
	local activeSpec = GetActiveSpecGroup()
	
	--will save to disk
	if ( activeSpec == 1 ) then --primary spec
		W_Options.Actions[dropSlot.slotNumber] = { dropSlot.theType, dropSlot.theID}; --the type is spell, item or macro. the id is spellID, itemID or macroID. store the settings in the saved variables table
	else --secondary spec
		W_Options.Actions2[dropSlot.slotNumber] = { dropSlot.theType, dropSlot.theID}
	end
	
end

--
-- We are dragging the item, spell or macro out of the slot. based on the type and id, copy the icon to the cursor
--
function WhammyConfig:DropSlot_OnDragStart( dropSlot )
	--Whammy:ShowMessage("WhammyConfig:DropSlot_OnDragStart")
	
	local enabled = dropSlot:IsEnabled() --returns true if enabled otherwise false

	if ( enabled == true ) then --only start a drag if the slot in enabled (it would be disabled if the item in the slot is not in the spellbook)

		local theType = dropSlot.theType
		local theID = dropSlot.theID

		--Whammy:ShowMessage("  dropSlot.theType: "..theType)
		--Whammy:ShowMessage("  dropSlot.theID: "..theID)

		WhammyConfig:DropSlot_AddThingToCursor( theType, theID)
		WhammyConfig:DropSlot_Clear(dropSlot);
	end

end

function WhammyConfig:ShowSlotInfo( dropSlot )

	if (dropSlot) then
		if (dropSlot.theType) then
			Whammy:ShowMessage("dropSlot.theType = "..dropSlot.theType)
		else
			Whammy:ShowMessage("dropSlot.theType = nil")
		end
		
		if ( dropSlot.theID ) then
			Whammy:ShowMessage("dropSlot.theID = "..dropSlot.theID)
		else
			Whammy:ShowMessage("dropSlot.theID = nil")
		end	
		
		if ( dropSlot.actionWhenPressed ) then
			Whammy:ShowMessage("dropSlot.actionWhenPressed = "..dropSlot.actionWhenPressed);
		else
			Whammy:ShowMessage("dropSlot.actionWhenPressed = nil")
		end
		
		if ( dropSlot.keyPress ) then
			Whammy:ShowMessage("dropSlot.keyPress = "..dropSlot.keyPress);
		else
			Whammy:ShowMessage("dropSlot.keyPress = nil")
		end
		
		if ( dropSlot.mouseButton ) then
			Whammy:ShowMessage("dropSlot.mouseButton = "..dropSlot.mouseButton);
		else
			Whammy:ShowMessage("dropSlot.mouseButton = nil")
		end
		

	else
		Whammy:ShowMessage("dropSlot was nil")
	end


end

-- Clear the drop slot when whatever was in it is dragged out
function WhammyConfig:DropSlot_Clear( dropSlot )
	--Whammy:ShowMessage("WhammyConfig:DropSlot_Clear: "..dropSlot:GetName() );

	dropSlot.theType = nil;
	dropSlot.theID = nil;
	dropSlot.actionWhenPressed = nil;
	dropSlot.locked = false
	SetItemButtonTexture( dropSlot, nil);
	
	dropSlot:Enable()
	
	local activeSpec = GetActiveSpecGroup(); --1 for primary spec, 2 for secondary
		
	if ( activeSpec == 1 ) then --primary spec
		W_Options.Actions[dropSlot.slotNumber] = nil;
	else --secondary spec
		W_Options.Actions2[dropSlot.slotNumber] = nil;
	end
	
	WhammyConfig:UpdateUnitButtons(dropSlot);
end

function WhammyConfig:DropSlot_SlotInit( dropSlot )

	dropSlot:RegisterForDrag("LeftButton")
	dropSlot:RegisterForClicks(nil) --we are just interested in leftClicks normally
	dropSlot:SetScript("OnReceiveDrag", function(self) WhammyConfig:DropSlot_Dropped(self); end );
	dropSlot:SetScript("OnClick", function(self) WhammyConfig:DropSlot_OnClick(self); end );
	dropSlot:SetScript("OnEnter", function(self) WhammyConfig:DropSlot_OnEnter(self); end);
	dropSlot:SetScript("OnLeave", function(self) WhammyConfig:DropSlot_OnLeave(self); end);
	dropSlot:SetScript("OnDragStart", function(self) WhammyConfig:DropSlot_OnDragStart(self); end);

end

--
-- called when we mouseover a disabled slot to give the user feedback about the item in the slot and enable right click to remove
--
function WhammyConfig:DropSlot_Disable(dropSlot)
	
	dropSlot:Enable() --need to enable it so use can right click to remove
	dropSlot.locked = true --set to locked so when the mouse leaves, the slot again disables
	GameTooltip:SetText("Item is not currently available\nPlease change stance, form or chakra to modify\nRight Click to remove");
	
	dropSlot:RegisterForDrag(nil) --cancel dragging
	
	dropSlot:RegisterForClicks("RightButtonUp") --we are just interested in right clicks at this point
	
	--only allow right clicking to remove the item in the slot
	dropSlot:SetScript("OnClick", function(self, button, down)
	  print("OnClick "..button)

		WhammyConfig:DropSlot_Clear(self) --clear the slot when we click the right button, also removed the thing from the saved var array
		WhammyConfig:DropSlot_SlotInit(self); --reset the slot to normal

	end)
	
end

--
-- when the cursor enters a dropSlot, set the tooltip to display what's in the slot
--
function WhammyConfig:DropSlot_OnEnter( dropSlot )
	GameTooltip:SetOwner(dropSlot, "ANCHOR_RIGHT");
	GameTooltip:ClearLines();
	
	if (dropSlot.theType) then --it has something in the slot
		local enabled = dropSlot:IsEnabled() --returns true if enabled otherwise false

		if ( enabled == false ) then
			WhammyConfig:DropSlot_Disable( dropSlot )

		else
			if ( dropSlot.theType == "item" ) then
				local itemID = dropSlot.theID;
				local name, link, quality, iLevel, reqLevel, class, subclass, maxStack, equipSlot, texture, vendorPrice = GetItemInfo(itemID)
				if ( name ) then
					GameTooltip:SetText(name);
				end

			elseif ( dropSlot.theType == "spell" ) then
				local spellID = dropSlot.theID;
				if ( spellID ) then
					GameTooltip:SetSpellByID(spellID);
				end

			elseif ( dropSlot.theType == "macro" ) then
				local macroID = dropSlot.data;
				local name, texture, body = GetMacroInfo(macroID)	
				if ( name ) then	
					GameTooltip:SetText(name);
				end

			end
		end


	end
end			

-- we leave the dropSlot, so clear the tooltip
function WhammyConfig:DropSlot_OnLeave( dropSlot )
	GameTooltip:ClearLines();
	GameTooltip:Hide();

	--the slot would be locked if the item in the slot is not avalable, like a chakra. If it's supposed to be disabled, then disable it upon mouse leaving
	if ( dropSlot.locked == true ) then
		dropSlot:Disable()
		WhammyConfig:DropSlot_SlotInit( dropSlot ) --now that the mouse has left the slot, reset it to normal events
	end
end

--
-- called when we click a dropSlot, treat like we want to pick up the item in the slot
--
function WhammyConfig:DropSlot_OnClick( dropSlot )
	Whammy:ShowMessage(dropSlot:GetName().." onclick");

	local theType, data, subType = GetCursorInfo(); --if we clicked something, then clicked the dropSlot, treat it like a drag and drop
	if ( theType ) then
		WhammyConfig:DropSlot_Dropped( dropSlot );
	end
end

--
--call to update all of the unit buttons when we have assigned a slot a new item, spell or macro
--
function WhammyConfig:UpdateUnitButtons( dropSlot )
	--Whammy:ShowMessage("UpdateUnitButtons: "..theDropSlot:GetName() )
	
	if ( InCombatLockdown() ) then
		--print("in combat")
		return
	end
	
	local buttonNum;
	local theType, actionWhenPressed;
	local keyPress = dropSlot.keyPress; --will be emptystring, shift, ctrl or alt
	local mouseButton = dropSlot.mouseButton; --will be a string of 1, 2, 3 etc

	if ( dropSlot.theType ) then
		theType = dropSlot.theType; --will be item spell or macro, or stop if we have reset or cleared the slot
		actionWhenPressed = dropSlot.actionWhenPressed; --will be the name of the item, spell or Macro
	else --slot was nil so clear the button out that is associated with the slot.
		theType = "stop"
		actionWhenPressed = nil;
	end
	
	if ( keyPress == "ctrl" or keyPress == "shift" or keyPress == "alt") then --if there is a keypress modifier then we need to add 
		keyPress = ( keyPress.."-" ); --                                       the dash so the attribute will be properly formatted
	end
	
	if ( theType == "companion" ) then --if the thing is a companion, we actually cast a spell to summon it so change the type to Spell
		theType = "spell"
	end
	
	local x = (keyPress.."type"..mouseButton);
	local y = (keyPress..theType..mouseButton);

	for buttonNum = 1,MAX_RAID_MEMBERS do
		local b = Whammy.buttonArray[buttonNum];
	
		b:SetAttribute( x, theType); --this attribute is the type spell, item etc.  paramters would be:  shifttype1, spell
		b:SetAttribute( y, actionWhenPressed); --this is the thing to perform on.  shiftspell1, Arcane Intellect
	end
end

--
-- Handle Clicks In Checkboxes
--
function WhammyConfig:Click_CheckButton_Summary( isChecked )
	local frame = WhammySummaryFrame
	if ( frame ) then
		if ( isChecked ) then
			frame:Show();
			W_Options.ShowSummary = true;
		else
			frame:Hide();
			W_Options.ShowSummary = false;
		end
	end
end

function WhammyConfig:Click_CheckButton_RangeCheck( isChecked ) --called when we actually click the button to enable range checking
	
	Whammy:RangeCheckToggle( isChecked );
	
end

function WhammyConfig:Click_CheckButton_ButtonSize( isChecked )
	if ( isChecked ) then
		W_Options.ButtonSize = Whammy.constants.kButtonSizeSmall;
	else
		W_Options.ButtonSize = Whammy.constants.kButtonSizeStandard;
	end
	
	Whammy:FrameDrawUnitButtons()
	Whammy:AdjustDragBarSize()
end


function WhammyConfig:Click_CheckButton_MuteAilmentSound( isChecked )
	if ( isChecked ) then
		W_Options.Mute = true;
	else
		W_Options.Mute = false;
	end
end

function WhammyConfig:Click_CheckButton_ThreatNotify( isChecked )
	if ( isChecked ) then
		W_Options.ThreatNotify = true;
	else
		W_Options.ThreatNotify = false;
	end
end

function WhammyConfig:Click_CheckButton_ShowDropdown( isChecked )
	if ( isChecked ) then
		W_Options.ShowDropdown = true;
	else
		W_Options.ShowDropdrown = false;
	end
end

function WhammyConfig:Click_CheckButton_IsFrameMovable( isChecked )

	if ( isChecked ) then
		W_Options.Movable = true;
	else
		W_Options.Movable = false;
	end
	
end



--
-- Default Button Stuff
--
function WhammyConfig:DefaultButton_OnClick() --clicking asks user if they are sure they want to reset back to defaults

	if ( StaticPopupDialogs["WHAMMY_DIALOG"] == nil ) then -- if we have not created this popup before, add it to the global popup table
		StaticPopupDialogs["WHAMMY_DIALOG"] = {
			text = "Are you sure you want to set popups to their defaults?",
			button1 = "Yes",
			button2 = "No",
			OnAccept = function()
				WhammyConfig:Reset_Whammy();
			end,
			timeout = 0,
			whileDead = 1,
			hideOnEscape = 1
		};
	end

	StaticPopup_Show("WHAMMY_DIALOG");
end

function WhammyConfig:Reset_Whammy()
	W_Options = nil; --blow out any settings
	
	Whammy:Init_Options(); --re-creates the table and set vars to default

	WhammyConfig:UpdateControlStatus(); --set the controls in the options frame to match the default values
	
	for slotNum = 1,20 do --clear out the drop slots
		local dropSlot = WhammyConfig.dropSlotArray[slotNum]
		WhammyConfig:DropSlot_Clear( dropSlot );
	end

	Whammy:Event_GroupRosterUpdate(); --set the frame to appropriate size for player or group

	Whammy:FrameDrawUnitButtons() --redraw the unit buttons in case the size changed
	Whammy:AdjustDragBarSize()

end


--
-- Called when we move the slider in the config window to move the miniMap icon around
-- 
function WhammyConfig:MiniMapIconPosSlider_ValueChanged( theSlider )
	W_Options.Angle = theSlider:GetValue();
	WhammyButton_Move();
end


--
-- toggle the config frame shown and hidden
--
function WhammyConfig:Config_Frame_Toggle()

	if ( whammyConfigFrame:IsVisible() ) then --hide the frame if it's visible
		whammyConfigFrame:Hide();
	else --show the frame if it's hidden
		whammyConfigFrame:Show();
	end

end

-- Whammy.Chakra[1] = 88625 --Chastise
-- Whammy.Chakra[2] = 88685 --holy word: Sanctuary
-- Whammy.Chakra[3] = 88684 --holy word: Serenity
-- Whammy.Chakra[4] = 20711 --Spirit of Redemption


--
-- pass in a globalSpellID, which is stored as a saved variable and this will return the slot it's in in the spellbook
--
function WhammyConfig:GetSpellBookIndexForGlobalSpellID( globalSpellID )
	--Whammy:ShowMessage("GetSpellBookIndexForGlobalSpellId")
	
	local spellNameGlobal, rank, icon, castingTime, minRange, maxRange, globalSpellID2 = GetSpellInfo(globalSpellID)
	
	--now iterage throgh the spellbook and find this spells index, and return it
	for index = 1, MAX_SPELLS do
		local spellNameSpellBook, spellSubName = GetSpellBookItemName(index, BOOKTYPE_SPELL)

		if ( spellNameGlobal == spellNameSpellBook ) then
			return index
		end
	end
	
	return nil
end

--
-- handle entering combat. If the frame is visible, close it
--

function WhammyConfig:HandleCombat()

	if ( whammyConfigFrame:IsVisible() ) then
		whammyConfigFrame.closeButton:Click()
	end
	
end

--
-- misc function to search a table for an element
--
function WhammyConfig.tableContains(table, element)
	for _, value in pairs(table) do
		if value == element then
		  return true
		end
	end
	
	return false
end
